//****************************************
// Variable deklarieren / Stand 29.02.24
//****************************************

#include <EEPROM.h>

int sensor_L = A1;     // Der Stromstärkesensor wird am Pin A1 (Analog "0") angeschlossen.
int sensor_R = A4;     // Der Stromstärkesensor wird am Pin A4 (Analog "0") angeschlossen.
int sensor_Kali = A3;  // Kalibrier Poti auf Pin A3

int sensorwert_L = 0;
int sensorwert_R = 0;
int sensorwert_Kali = 0;

int merker_kali_links = 0;
int merker_kali_rechts = 0;
int zaehler_reed = 0;
int zaehler_kurbel = 0;
int Puls = 0;

unsigned long previousMillis = 0;
unsigned long currentMillis;
long intervalon = 600;  //1,5 Sekunden
long intervaloff = 650;
const long intervalmess = 50;
boolean buttonState, oldButtonState;
unsigned long anzeigeMillis, aktMillis;
const int anzeigeZeit = 1000;

int buttonApin = 2;   //digitaler Eingang / Links
int buttonBpin = 3;   //digitaler Eingang / Rechts
int buttonCpin = 4;   //digitaler Eingang / Licht
int buttonDpin = 5;   //digitaler Eingang / Hupe
int buttonEpin = 6;   //digitaler Eingang / Reed
int buttonFpin = 13;  //digitaler Eingang / Kalibrieren on/off

enum States { stateRuhestellung,
              stateWarte,
              stateI_messen,
              stateBlink_off,
              stateEnde };
States mState = stateRuhestellung;  //der Startpunkt der Schrittkette

enum Reed { stateReed0,
            stateReed1,
            stateReed2,
            stateReed3,
            stateReedEnde };
Reed mReed = stateReed0;  //der Startpunkt der Schrittkette

bool einmal_Blink = true;
bool einmal_Reed = true;
bool immer_null = false;
bool test;
bool start_mess;  //Merker um Messung zu starten

bool I_BlinkLinks;   //Eingang Blinker links
bool I_BlinkRechts;  //Eingang Blinker rechts
bool I_Licht;        //Eingang Licht
bool I_Hupe;         //Eingang Hupe
bool I_Reed;         //Eingang Reedkontakt
bool I_Kali;         //Eingang Kalibrieren

//******************************
// Setup / Einmal durchlaufen
//******************************

void setup() {
  Serial.begin(9600);  // Verwende die serielle Schnittstelle, um die Nummer auszugeben

  pinMode(buttonApin, INPUT_PULLUP);  //digitaler Eingang / Links
  pinMode(buttonBpin, INPUT_PULLUP);  //digitaler Eingang / Rechts
  pinMode(buttonCpin, INPUT_PULLUP);  //digitaler Eingang / Licht
  pinMode(buttonDpin, INPUT_PULLUP);  //digitaler Eingang / Hupe
  pinMode(buttonEpin, INPUT_PULLUP);  //digitaler Eingang / Reedkontakt
  pinMode(buttonFpin, INPUT_PULLUP);  //digitaler Eingang / Kalibrieren

  pinMode(7, OUTPUT);   // Blinker Links
  pinMode(8, OUTPUT);   // Blinker Rechts
  pinMode(9, OUTPUT);   // Blink LED
  pinMode(10, OUTPUT);  // Reed Relais
  pinMode(11, OUTPUT);  // Licht
  pinMode(12, OUTPUT);  // Hupe

//  pinMode(LED_BUILTIN, OUTPUT);

  EEPROM.get(0, merker_kali_links);
  EEPROM.get(2, merker_kali_rechts);
}

//******************************
// Zyklisches Programm
//******************************

void loop() {

  sensorwert_L = analogRead(sensor_L);
  sensorwert_R = analogRead(sensor_R);
  sensorwert_Kali = analogRead(sensor_Kali);

  unsigned long currentMillis = millis();

  I_BlinkLinks = !digitalRead(buttonApin);
  I_BlinkRechts = !digitalRead(buttonBpin);
  I_Licht = !digitalRead(buttonCpin);
  I_Hupe = !digitalRead(buttonDpin);
  I_Reed = !digitalRead(buttonEpin);
  I_Kali = !digitalRead(buttonFpin);

  //*************************************************
  // Schrittkette Blinker                           *
  //*************************************************

  // ************************************************
  // * Blinker / Schritt 0 / Blinker einschalten    *
  // ************************************************
  switch (mState) {
    case stateRuhestellung:
      {
        if (einmal_Blink) {
          einmal_Blink = false;
          digitalWrite(7, LOW);
          digitalWrite(8, LOW);
          digitalWrite(9, LOW);
        }
        if (I_BlinkLinks == HIGH) {
          einmal_Blink = true;
          digitalWrite(7, HIGH);  // Relais Blinker Links
          digitalWrite(9, HIGH);  // Relais Blinker LED
          mState = stateWarte;    // der nächste Schritt
        }
        if (I_BlinkRechts == HIGH) {
          einmal_Blink = true;
          digitalWrite(8, HIGH);  // Relais Blinker Rechts
          digitalWrite(9, HIGH);  // Relais Blinker LED
          mState = stateWarte;    // der nächste Schritt
        }
      }
      break;

      // ************************************************
      // * Blinker / Schritt 1 / Warte bis Strom konst. *
      // ************************************************

    case stateWarte:
      {
        if (einmal_Blink) {
          einmal_Blink = false;
        }
        if (currentMillis - previousMillis > intervalmess) {
          previousMillis = currentMillis;
          einmal_Blink = true;
          mState = stateI_messen;
          start_mess = true;
        }
      }
      break;

      // ************************************************
      // * Blinker / Schritt 2 / Strom mess, Ergebnis ? *
      // ************************************************

    case stateI_messen:
      {
        if (einmal_Blink) {
          einmal_Blink = false;
        }

        if ((I_BlinkLinks == HIGH && sensorwert_L < merker_kali_links) || (I_BlinkRechts == HIGH && sensorwert_R < merker_kali_rechts)) {  // Stromwert wenn Lampe an
          intervalon = 250;                                                                                                                //250
          intervaloff = 300;                                                                                                               //300
        } else {
          intervalon = 600;   //600
          intervaloff = 650;  //650
        }

        if (currentMillis - previousMillis > intervalon) {
          previousMillis = currentMillis;
          einmal_Blink = true;

          if (I_BlinkLinks == HIGH) {
            Serial.print("sensorwert_L:  ");
            Serial.print(sensorwert_L);
            Serial.print(" / ");

            Serial.print("merker_kali_links:  ");
            Serial.print(merker_kali_links);
            Serial.print(" / ");

            Serial.print("I_Kali:  ");
            Serial.print(I_Kali);
            Serial.print(" / ");

            Serial.print("sensorwert_Kali:  ");
            Serial.print(sensorwert_Kali);
            Serial.println(" / ");
          }

          if (I_BlinkRechts == HIGH) {
            Serial.print("sensorwert_R:  ");
            Serial.print(sensorwert_R);
            Serial.print(" / ");

            Serial.print("merker_kali_rechts:  ");
            Serial.print(merker_kali_rechts);
            Serial.print(" / ");

            Serial.print("I_Kali:  ");
            Serial.print(I_Kali);
            Serial.print(" / ");

            Serial.print("sensorwert_Kali:  ");
            Serial.print(sensorwert_Kali);
            Serial.println(" / ");
          }

          start_mess = false;
          mState = stateBlink_off;
        }
      }
      break;

      // ************************************************
      // * Blinker / Schritt 3 / Blinker aus            *
      // ************************************************

    case stateBlink_off:
      {
        if (einmal_Blink) {
          einmal_Blink = false;
          digitalWrite(7, LOW);  // Relais Blinker Links
          digitalWrite(8, LOW);  // Relais Blinker Rechts
          digitalWrite(9, LOW);  // Relais Blinker LED
        }
        if (currentMillis - previousMillis > intervaloff) {
          previousMillis = currentMillis;
          einmal_Blink = true;
          mState = stateEnde;
        }
      }
      break;

    // ************************************************
    // * Blinker / Schritt 4 / Letzter Schritt        *
    // ************************************************
    case stateEnde:
      {
        if (einmal_Blink) {
          einmal_Blink = false;
        }
        if (1 == 1) {
          einmal_Blink = true;
          mState = stateRuhestellung;
        }
      }
      break;
  }

  //*************************************************
  // Schrittkette Reedkontakt                       *
  //*************************************************

  // ************************************************
  // * Reed / Schritt 0 / Reedkontakt High          *
  // ************************************************
  switch (mReed) {
    case stateReed0:
      {
        if (einmal_Reed) {
        }
        if (I_Reed == HIGH) {
          einmal_Reed = true;
          Puls = Puls + 1;
          zaehler_kurbel = 90;
          mReed = stateReed1;  // der nächste Schritt
        }

        if (Puls > 4) {
          einmal_Reed = true;
          zaehler_reed = 3000;
          mReed = stateReed2;  // Daumengas freigeben
        }
      }
      break;

      // ************************************************
      // * Reed / Schritt 1 / Reedkontakt Low           *
      // ************************************************

    case stateReed1:
      {
        if (einmal_Reed) {
        }
        if (I_Reed == LOW) {
          einmal_Reed = true;
          mReed = stateReed0;  // der nächste Schritt
        }
      }
      break;

      // ************************************************
      // * Reed / Schritt 2 / Daumengas freigeben       *
      // ************************************************

    case stateReed2:
      {
        if (einmal_Reed) {
        }
        if (I_Reed == HIGH) {
          einmal_Reed = true;
          digitalWrite(10, HIGH);  // Relais Reed ein
          Puls = 0;
          mReed = stateReed3;  // der nächste Schritt
        }
      }
      break;


      // ************************************************
      // * Reed / Schritt 3 / Dauemgas sperren          *
      // ************************************************

    case stateReed3:
      {
        if (einmal_Reed) {
        }
        if (zaehler_reed == 0) {
          einmal_Reed = true;
          digitalWrite(10, LOW);  // Relais Reed aus
          mReed = stateReedEnde;  // der nächste Schritt
        }
        if (I_Reed == HIGH) {
          zaehler_reed = 300;
        }
      }
      break;

    // ************************************************
    // * Reed / Schritt 4 / Letzter Schritt           *
    // ************************************************
    case stateReedEnde:
      {
        if (einmal_Reed) {
          einmal_Reed = true;
          mReed = stateReed0;  // Springe an Anfang
        }
      }
      break;
  }

  //*************************************************
  // Zyklisch ausserhalb der Kette                  *
  //*************************************************

  // ************************************************
  // * Reed Zähler                                  *
  // ************************************************

  if (zaehler_kurbel > 0) {
    zaehler_kurbel = zaehler_kurbel - 1;
  } else {
    Puls = 0;
  }

  if (zaehler_reed > 0) {
    zaehler_reed = zaehler_reed - 1;
  } 

  // ************************************************
  // * Kalibrierwert in EEPROM speichern            *
  // ************************************************

  if (I_Kali == HIGH && I_BlinkLinks == HIGH) {
    merker_kali_links = sensorwert_Kali;  // Poti an Merker übergeben
    EEPROM.put(0, merker_kali_links);
  }

  if (I_Kali == HIGH && I_BlinkRechts == HIGH) {
    merker_kali_rechts = sensorwert_Kali;  // Poti an Merker übergeben
    EEPROM.put(2, merker_kali_rechts);
  }

  // ************************************************
  // * Licht                                        *
  // ************************************************

  if (I_Licht == HIGH) {
    digitalWrite(11, HIGH);  // Relais Licht ein
  } else {
    digitalWrite(11, LOW);  // Relais Licht aus
  }

  // ************************************************
  // * Hupe                                         *
  // ************************************************

  if (I_Hupe == HIGH) {
    digitalWrite(12, HIGH);  // Relais Hupe ein
  } else {
    digitalWrite(12, LOW);  // Relais Hupe aus
  }

  // ************************************************
  // * Diagnose                                     *
  // ************************************************

  Serial.print("Puls:  ");
  Serial.print(Puls);

  Serial.print(" / ");

  Serial.print("mReed:  ");
  Serial.print(mReed);

  Serial.print(" / ");

  Serial.print("zaehler_kurbel:  ");
  Serial.print(zaehler_kurbel);

  Serial.print(" / ");

  Serial.print("zaehler_reed:  ");
  Serial.print(zaehler_reed);

  Serial.println(" / ");

  //******************************
  // Programm Ende
  //******************************
}
